From: Yamin Friedman <yam...@mellanox.com>

If there are more scatter entries than the recommended limit provided by
the ib device, UMR registration is used. This will provide optimal
performance when performing large RDMA READs over devices that advertise
the threshold capability.

With ConnectX-5 running NVMeoF RDMA with FIO single QP 128KB writes:
Without use of cap: 70Gb/sec
With use of cap: 84Gb/sec

Signed-off-by: Yamin Friedman <yam...@mellanox.com>
Reviewed-by: Or Gerlitz <ogerl...@mellanox.com>
Signed-off-by: Leon Romanovsky <leo...@mellanox.com>
---
 drivers/infiniband/core/rw.c | 25 +++++++++++++++----------
 include/rdma/ib_verbs.h      |  2 ++
 2 files changed, 17 insertions(+), 10 deletions(-)

diff --git a/include/rdma/ib_verbs.h b/include/rdma/ib_verbs.h
index 4f671378dbfc..60fd98a9b7e8 100644
--- a/include/rdma/ib_verbs.h
+++ b/include/rdma/ib_verbs.h
@@ -445,6 +445,8 @@ struct ib_device_attr {
        struct ib_tm_caps       tm_caps;
        struct ib_cq_caps       cq_caps;
        u64                     max_dm_size;
+       /* Max entries for sgl for optimized performance per READ */
+       u32                     max_sgl_rd;
 };

 enum ib_mtu {
diff --git a/drivers/infiniband/core/rw.c b/drivers/infiniband/core/rw.c
index 5337393d4dfe..c27a543b58ef 100644
--- a/drivers/infiniband/core/rw.c
+++ b/drivers/infiniband/core/rw.c
@@ -20,14 +20,16 @@ module_param_named(force_mr, rdma_rw_force_mr, bool, 0);
 MODULE_PARM_DESC(force_mr, "Force usage of MRs for RDMA READ/WRITE 
operations");

 /*
  * Check if the device might use memory registration.  This is currently only
  * true for iWarp devices. In the future we can hopefully fine tune this based
  * on HCA driver input.
  */
 static inline bool rdma_rw_can_use_mr(struct ib_device *dev, u8 port_num)
 {
        if (rdma_protocol_iwarp(dev, port_num))
                return true;
+       if (dev->attrs.max_sgl_rd)
+               return true;
        if (unlikely(rdma_rw_force_mr))
                return true;
        return false;
@@ -35,17 +37,20 @@ static inline bool rdma_rw_can_use_mr(struct ib_device 
*dev, u8 port_num)

 /*
  * Check if the device will use memory registration for this RW operation.
- * We currently always use memory registrations for iWarp RDMA READs, and
- * have a debug option to force usage of MRs.
- *
- * XXX: In the future we can hopefully fine tune this based on HCA driver
- * input.
+ * For RDMA READs we must use MRs on iWarp and can optionaly use them as an
+ * optimaztion otherwise.  Additionally we have a debug option to force usage
+ * of MRs to help testing this code path.
  */
+
 static inline bool rdma_rw_io_needs_mr(struct ib_device *dev, u8 port_num,
                enum dma_data_direction dir, int dma_nents)
 {
-       if (rdma_protocol_iwarp(dev, port_num) && dir == DMA_FROM_DEVICE)
-               return true;
+       if (dir == DMA_FROM_DEVICE) {
+               if (rdma_protocol_iwarp(dev, port_num))
+                       return true;
+               if (dev->attrs.max_sgl_rd && dma_nents > dev->attrs.max_sgl_rd)
+                       return true;
+       }
        if (unlikely(rdma_rw_force_mr))
                return true;
        return false;
@@ -20,14 +20,16 @@ module_param_named(force_mr, rdma_rw_force_mr, bool, 0);

 /*
- * Check if the device might use memory registration.  This is currently only
- * true for iWarp devices. In the future we can hopefully fine tune this based
- * on HCA driver input.
+ * Check if the device might use memory registration. This is currently
+ * true for iWarp devices and devices that have optimized SGL registration
+ * logic.
  */
 static inline bool rdma_rw_can_use_mr(struct ib_device *dev, u8 port_num)
--
2.20.1

Reply via email to